@@ -5,10 +5,9 @@ import numpy as np
 
 from pandas.core.algorithms import factorize, match, unique, value_counts
 from pandas.core.common import isnull, notnull
-from pandas.core.categorical import Categorical, Factor
-from pandas.core.format import (set_printoptions, reset_printoptions,
-                                set_eng_float_format)
-from pandas.core.index import Index, Int64Index, MultiIndex
+from pandas.core.categorical import Categorical
+from pandas.core.format import set_eng_float_format
+from pandas.core.index import Index, Int64Index, Float64Index, MultiIndex
 
 from pandas.core.series import Series, TimeSeries
 from pandas.core.frame import DataFrame
@@ -16,7 +15,7 @@ from pandas.core.panel import Panel
 from pandas.core.panel4d import Panel4D
 from pandas.core.groupby import groupby
 from pandas.core.reshape import (pivot_simple as pivot, get_dummies,
-                                 lreshape)
+                                 lreshape, wide_to_long)
 
 WidePanel = Panel
 
@@ -28,8 +27,8 @@ from pandas.tseries.period import Period, PeriodIndex
 
 # legacy
 from pandas.core.daterange import DateRange  # deprecated
-from pandas.core.common import save, load # deprecated, remove in 0.13
+from pandas.core.common import save, load  # deprecated, remove in 0.13
 import pandas.core.datetools as datetools
 
-from pandas.core.config import get_option, set_option, reset_option,\
-    describe_option, options
+from pandas.core.config import (get_option, set_option, reset_option,
+                                describe_option, options)
