@@ -1,9 +1,7 @@
 """
 The config module holds package-wide configurables and provides
 a uniform API for working with them.
-"""
 
-"""
 Overview
 ========
 
@@ -54,6 +52,8 @@ import re
 
 from collections import namedtuple
 import warnings
+from pandas.compat import map, lmap, u
+import pandas.compat as compat
 
 DeprecatedOption = namedtuple('DeprecatedOption', 'key msg rkey removal_ver')
 RegisteredOption = namedtuple(
@@ -65,7 +65,13 @@ _global_config = {}  # holds the current values for registered options
 _reserved_keys = ['all']  # keys which have a special meaning
 
 
-##########################################
+class OptionError(AttributeError, KeyError):
+
+    """Exception for pandas.options, backwards compatible with KeyError
+    checks"""
+
+
+#
 # User API
 
 def _get_single_key(pat, silent):
@@ -73,9 +79,9 @@ def _get_single_key(pat, silent):
     if len(keys) == 0:
         if not silent:
             _warn_if_deprecated(pat)
-        raise KeyError('No such keys(s)')
+        raise OptionError('No such keys(s): %r' % pat)
     if len(keys) > 1:
-        raise KeyError('Pattern matched multiple keys')
+        raise OptionError('Pattern matched multiple keys')
     key = keys[0]
 
     if not silent:
@@ -94,7 +100,7 @@ def _get_option(pat, silent=False):
     return root[k]
 
 
-def _set_option(pat, value, silent=False):
+def _set_single_option(pat, value, silent):
     key = _get_single_key(pat, silent)
 
     o = _get_registered_option(key)
@@ -109,18 +115,52 @@ def _set_option(pat, value, silent=False):
         o.cb(key)
 
 
+def _set_multiple_options(args, silent):
+    for k, v in zip(args[::2], args[1::2]):
+        _set_single_option(k, v, silent)
+
+
+def _set_option(*args, **kwargs):
+    # must at least 1 arg deal with constraints later
+    nargs = len(args)
+    if not nargs or nargs % 2 != 0:
+        raise AssertionError("Must provide an even number of non-keyword "
+                             "arguments")
+
+    # must be 0 or 1 kwargs
+    nkwargs = len(kwargs)
+    if nkwargs not in (0, 1):
+        raise AssertionError("The can only be 0 or 1 keyword arguments")
+
+    # if 1 kwarg then it must be silent=True or silent=False
+    if nkwargs:
+        k, = list(kwargs.keys())
+        v, = list(kwargs.values())
+
+        if k != 'silent':
+            raise ValueError("the only allowed keyword argument is 'silent', "
+                             "you passed '{0}'".format(k))
+        if not isinstance(v, bool):
+            raise TypeError("the type of the keyword argument passed must be "
+                            "bool, you passed a {0}".format(v.__class__))
+
+    # default to false
+    silent = kwargs.get('silent', False)
+    _set_multiple_options(args, silent)
+
+
 def _describe_option(pat='', _print_desc=True):
 
     keys = _select_options(pat)
     if len(keys) == 0:
-        raise KeyError('No such keys(s)')
+        raise OptionError('No such keys(s)')
 
-    s = u''
+    s = u('')
     for k in keys:  # filter by pat
         s += _build_option_description(k)
 
     if _print_desc:
-        print s
+        print(s)
     else:
         return s
 
@@ -130,20 +170,28 @@ def _reset_option(pat):
     keys = _select_options(pat)
 
     if len(keys) == 0:
-        raise KeyError('No such keys(s)')
+        raise OptionError('No such keys(s)')
 
     if len(keys) > 1 and len(pat) < 4 and pat != 'all':
         raise ValueError('You must specify at least 4 characters when '
-                         'resetting multiple keys, use the special keyword "all" '
-                         'to reset all the options to their default value')
+                         'resetting multiple keys, use the special keyword '
+                         '"all" to reset all the options to their default '
+                         'value')
 
     for k in keys:
         _set_option(k, _registered_options[k].defval)
 
 
+def get_default_val(pat):
+    key = _get_single_key(pat, silent=True)
+    return _get_registered_option(key).defval
+
+
 class DictWrapper(object):
+
     """ provide attribute-style access to a nested dict
     """
+
     def __init__(self, d, prefix=""):
         object.__setattr__(self, "d", d)
         object.__setattr__(self, "prefix", prefix)
@@ -158,7 +206,7 @@ class DictWrapper(object):
         if key in self.d and not isinstance(self.d[key], dict):
             _set_option(prefix, val)
         else:
-            raise KeyError("You can only set the value of existing options")
+            raise OptionError("You can only set the value of existing options")
 
     def __getattr__(self, key):
         prefix = object.__getattribute__(self, "prefix")
@@ -172,7 +220,8 @@ class DictWrapper(object):
             return _get_option(prefix)
 
     def __dir__(self):
-        return self.d.keys()
+        return list(self.d.keys())
+
 
 # For user convenience,  we'd like to have the available options described
 # in the docstring. For dev convenience we'd like to generate the docstrings
@@ -183,7 +232,7 @@ class DictWrapper(object):
 # of options, and option descriptions.
 
 
-class CallableDyanmicDoc(object):
+class CallableDynamicDoc(object):
 
     def __init__(self, func, doc_tmpl):
         self.__doc_tmpl__ = doc_tmpl
@@ -195,11 +244,12 @@ class CallableDyanmicDoc(object):
     @property
     def __doc__(self):
         opts_desc = _describe_option('all', _print_desc=False)
-        opts_list = pp_options_list(_registered_options.keys())
+        opts_list = pp_options_list(list(_registered_options.keys()))
         return self.__doc_tmpl__.format(opts_desc=opts_desc,
                                         opts_list=opts_list)
 
-_get_option_tmpl = """"get_option(pat) - Retrieves the value of the specified option
+_get_option_tmpl = """
+get_option(pat) - Retrieves the value of the specified option
 
 Available options:
 {opts_list}
@@ -218,12 +268,13 @@ result - the value of the option
 
 Raises
 ------
-KeyError if no such option exists
+OptionError if no such option exists
 
 {opts_desc}
 """
 
-_set_option_tmpl = """set_option(pat,value) - Sets the value of the specified option
+_set_option_tmpl = """
+set_option(pat,value) - Sets the value of the specified option
 
 Available options:
 {opts_list}
@@ -244,12 +295,13 @@ None
 
 Raises
 ------
-KeyError if no such option exists
+OptionError if no such option exists
 
 {opts_desc}
 """
 
-_describe_option_tmpl = """describe_option(pat,_print_desc=False) Prints the description
+_describe_option_tmpl = """
+describe_option(pat,_print_desc=False) Prints the description
 for one or more registered options.
 
 Call with not arguments to get a listing for all registered options.
@@ -274,7 +326,8 @@ is False
 {opts_desc}
 """
 
-_reset_option_tmpl = """reset_option(pat) - Reset one or more options to their default value.
+_reset_option_tmpl = """
+reset_option(pat) - Reset one or more options to their default value.
 
 Pass "all" as argument to reset all options.
 
@@ -298,21 +351,26 @@ None
 
 # bind the functions with their docstrings into a Callable
 # and use that as the functions exposed in pd.api
-get_option = CallableDyanmicDoc(_get_option, _get_option_tmpl)
-set_option = CallableDyanmicDoc(_set_option, _set_option_tmpl)
-reset_option = CallableDyanmicDoc(_reset_option, _reset_option_tmpl)
-describe_option = CallableDyanmicDoc(_describe_option, _describe_option_tmpl)
+get_option = CallableDynamicDoc(_get_option, _get_option_tmpl)
+set_option = CallableDynamicDoc(_set_option, _set_option_tmpl)
+reset_option = CallableDynamicDoc(_reset_option, _reset_option_tmpl)
+describe_option = CallableDynamicDoc(_describe_option, _describe_option_tmpl)
 options = DictWrapper(_global_config)
 
-######################################################
+#
 # Functions for use by pandas developers, in addition to User - api
 
 
 class option_context(object):
+
     def __init__(self, *args):
-        assert len(args) % 2 == 0 and len(args) >= 2, \
-            "Need to invoke as option_context(pat,val,[(pat,val),..))."
-        ops = zip(args[::2], args[1::2])
+        if not (len(args) % 2 == 0 and len(args) >= 2):
+            raise AssertionError(
+                'Need to invoke as'
+                'option_context(pat, val, [(pat, val), ...)).'
+            )
+
+        ops = list(zip(args[::2], args[1::2]))
         undo = []
         for pat, val in ops:
             undo.append((pat, _get_option(pat, silent=True)))
@@ -359,9 +417,9 @@ def register_option(key, defval, doc='', validator=None, cb=None):
     key = key.lower()
 
     if key in _registered_options:
-        raise KeyError("Option '%s' has already been registered" % key)
+        raise OptionError("Option '%s' has already been registered" % key)
     if key in _reserved_keys:
-        raise KeyError("Option '%s' is a reserved key" % key)
+        raise OptionError("Option '%s' is a reserved key" % key)
 
     # the default value should be legal
     if validator:
@@ -379,21 +437,22 @@ def register_option(key, defval, doc='', validator=None, cb=None):
     cursor = _global_config
     for i, p in enumerate(path[:-1]):
         if not isinstance(cursor, dict):
-            raise KeyError("Path prefix to option '%s' is already an option"
-                           % '.'.join(path[:i]))
+            raise OptionError("Path prefix to option '%s' is already an option"
+                              % '.'.join(path[:i]))
         if p not in cursor:
             cursor[p] = {}
         cursor = cursor[p]
 
     if not isinstance(cursor, dict):
-        raise KeyError("Path prefix to option '%s' is already an option"
-                       % '.'.join(path[:-1]))
+        raise OptionError("Path prefix to option '%s' is already an option"
+                          % '.'.join(path[:-1]))
 
     cursor[path[-1]] = defval  # initialize
 
     # save the option metadata
     _registered_options[key] = RegisteredOption(key=key, defval=defval,
-                                                doc=doc, validator=validator, cb=cb)
+                                                doc=doc, validator=validator,
+                                                cb=cb)
 
 
 def deprecate_option(key, msg=None, rkey=None, removal_ver=None):
@@ -431,20 +490,20 @@ def deprecate_option(key, msg=None, rkey=None, removal_ver=None):
 
     Raises
     ------
-    KeyError - if key has already been deprecated.
+    OptionError - if key has already been deprecated.
 
     """
 
     key = key.lower()
 
     if key in _deprecated_options:
-        raise KeyError("Option '%s' has already been defined as deprecated."
-                       % key)
+        raise OptionError("Option '%s' has already been defined as deprecated."
+                          % key)
 
     _deprecated_options[key] = DeprecatedOption(key, msg, rkey, removal_ver)
 
 
-################################
+#
 # functions internal to the module
 
 def _select_options(pat):
@@ -453,6 +512,11 @@ def _select_options(pat):
     if pat=="all", returns all registered options
     """
 
+    # short-circuit for exact key
+    if pat in _registered_options:
+        return [pat]
+
+    # else look through all of them
     keys = sorted(_registered_options.keys())
     if pat == 'all':  # reserved key
         return keys
@@ -500,13 +564,7 @@ def _get_registered_option(key):
     -------
     RegisteredOption (namedtuple) if key is deprecated, None otherwise
     """
-
-    try:
-        d = _registered_options[key]
-    except KeyError:
-        return None
-    else:
-        return d
+    return _registered_options.get(key)
 
 
 def _translate_key(key):
@@ -554,16 +612,21 @@ def _build_option_description(k):
 
     o = _get_registered_option(k)
     d = _get_deprecated_option(k)
-    s = u'%s: ' % k
+
+    s = u('%s: ') % k
+    if o:
+        s += u('[default: %s] [currently: %s]') % (o.defval,
+                                                   _get_option(k, True))
+
     if o.doc:
         s += '\n' + '\n    '.join(o.doc.strip().split('\n'))
     else:
         s += 'No description available.\n'
 
     if d:
-        s += u'\n\t(Deprecated'
-        s += (u', use `%s` instead.' % d.rkey if d.rkey else '')
-        s += u')\n'
+        s += u('\n\t(Deprecated')
+        s += (u(', use `%s` instead.') % d.rkey if d.rkey else '')
+        s += u(')\n')
 
     s += '\n'
     return s
@@ -594,12 +657,12 @@ def pp_options_list(keys, width=80, _print=False):
         ls += pp(k, ks)
     s = '\n'.join(ls)
     if _print:
-        print s
+        print(s)
     else:
         return s
 
 
-##############
+#
 # helpers
 
 from contextlib import contextmanager
@@ -692,19 +755,35 @@ def is_instance_factory(_type):
                 True if x is an instance of `_type`
 
     """
+    if isinstance(_type, (tuple, list)):
+        _type = tuple(_type)
+        from pandas.core.common import pprint_thing
+        type_repr = "|".join(map(pprint_thing, _type))
+    else:
+        type_repr = "'%s'" % _type
 
     def inner(x):
         if not isinstance(x, _type):
-            raise ValueError("Value must be an instance of '%s'" % str(_type))
+            raise ValueError("Value must be an instance of %s" % type_repr)
 
     return inner
 
 
+def is_one_of_factory(legal_values):
+    def inner(x):
+        from pandas.core.common import pprint_thing as pp
+        if not x in legal_values:
+            pp_values = lmap(pp, legal_values)
+            raise ValueError("Value must be one of %s"
+                             % pp("|".join(pp_values)))
+
+    return inner
+
 # common type validators, for convenience
 # usage: register_option(... , validator = is_int)
 is_int = is_type_factory(int)
 is_bool = is_type_factory(bool)
 is_float = is_type_factory(float)
 is_str = is_type_factory(str)
-is_unicode = is_type_factory(unicode)
-is_text = is_instance_factory(basestring)
+is_unicode = is_type_factory(compat.text_type)
+is_text = is_instance_factory((str, bytes))
