@@ -1,10 +1,13 @@
 import numpy as np
 
-from itertools import izip
-from pandas.core.common import isnull
+from pandas.compat import zip
+from pandas.core.common import isnull, _values_from_object
 from pandas.core.series import Series
+from pandas.core.frame import DataFrame
+import pandas.compat as compat
 import re
 import pandas.lib as lib
+import warnings
 
 
 def _get_array_list(arr, others):
@@ -50,7 +53,7 @@ def str_cat(arr, others=None, sep=None, na_rep=None):
 
             notmask = -na_mask
 
-            tuples = izip(*[x[notmask] for x in arrays])
+            tuples = zip(*[x[notmask] for x in arrays])
             cats = [sep.join(tup) for tup in tuples]
 
             result[notmask] = cats
@@ -90,6 +93,8 @@ def _na_map(f, arr, na_result=np.nan):
 
 
 def _map(f, arr, na_mask=False, na_value=np.nan):
+    if isinstance(arr, Series):
+        arr = arr.values
     if not isinstance(arr, np.ndarray):
         arr = np.asarray(arr, dtype=object)
     if na_mask:
@@ -112,6 +117,17 @@ def _map(f, arr, na_mask=False, na_value=np.nan):
         return lib.map_infer(arr, f)
 
 
+def str_title(arr):
+    """
+    Convert strings to titlecased version
+
+    Returns
+    -------
+    titled : array
+    """
+    return _na_map(lambda x: x.title(), arr)
+
+
 def str_count(arr, pat, flags=0):
     """
     Count occurrences of pattern in each string
@@ -144,7 +160,7 @@ def str_contains(arr, pat, case=True, flags=0, na=np.nan):
         If True, case sensitive
     flags : int, default 0 (no flags)
         re module flags, e.g. re.IGNORECASE
-    na : bool, default NaN
+    na : default NaN, fill value for missing values.
 
     Returns
     -------
@@ -155,6 +171,10 @@ def str_contains(arr, pat, case=True, flags=0, na=np.nan):
 
     regex = re.compile(pat, flags=flags)
 
+    if regex.groups > 0:
+        warnings.warn("This pattern has match groups. To actually get the"
+                      " groups, use str.extract.", UserWarning)
+
     f = lambda x: bool(regex.search(x))
     return _na_map(f, arr, na)
 
@@ -272,24 +292,87 @@ def str_repeat(arr, repeats):
     if np.isscalar(repeats):
         def rep(x):
             try:
-                return str.__mul__(x, repeats)
+                return compat.binary_type.__mul__(x, repeats)
             except TypeError:
-                return unicode.__mul__(x, repeats)
+                return compat.text_type.__mul__(x, repeats)
+
         return _na_map(rep, arr)
     else:
         def rep(x, r):
             try:
-                return str.__mul__(x, r)
+                return compat.binary_type.__mul__(x, r)
             except TypeError:
-                return unicode.__mul__(x, r)
+                return compat.text_type.__mul__(x, r)
+
         repeats = np.asarray(repeats, dtype=object)
-        result = lib.vec_binop(arr, repeats, rep)
+        result = lib.vec_binop(_values_from_object(arr), repeats, rep)
         return result
 
 
-def str_match(arr, pat, flags=0):
+def str_match(arr, pat, case=True, flags=0, na=np.nan, as_indexer=False):
     """
-    Find groups in each string (from beginning) using passed regular expression
+    Deprecated: Find groups in each string using passed regular expression.
+    If as_indexer=True, determine if each string matches a regular expression.
+
+    Parameters
+    ----------
+    pat : string
+        Character sequence or regular expression
+    case : boolean, default True
+        If True, case sensitive
+    flags : int, default 0 (no flags)
+        re module flags, e.g. re.IGNORECASE
+    na : default NaN, fill value for missing values.
+    as_indexer : False, by default, gives deprecated behavior better achieved
+        using str_extract. True return boolean indexer.
+
+
+    Returns
+    -------
+    matches : boolean array (if as_indexer=True)
+    matches : array of tuples (if as_indexer=False, default but deprecated)
+
+    Notes
+    -----
+    To extract matched groups, which is the deprecated behavior of match, use
+    str.extract.
+    """
+
+    if not case:
+        flags |= re.IGNORECASE
+
+    regex = re.compile(pat, flags=flags)
+
+    if (not as_indexer) and regex.groups > 0:
+        # Do this first, to make sure it happens even if the re.compile
+        # raises below.
+        warnings.warn("In future versions of pandas, match will change to"
+                      " always return a bool indexer.""", UserWarning)
+
+    if as_indexer and regex.groups > 0:
+        warnings.warn("This pattern has match groups. To actually get the"
+                      " groups, use str.extract.""", UserWarning)
+
+    # If not as_indexer and regex.groups == 0, this returns empty lists
+    # and is basically useless, so we will not warn.
+
+    if (not as_indexer) and regex.groups > 0:
+        def f(x):
+            m = regex.match(x)
+            if m:
+                return m.groups()
+            else:
+                return []
+    else:
+        # This is the new behavior of str_match.
+        f = lambda x: bool(regex.match(x))
+
+    return _na_map(f, arr)
+
+
+def str_extract(arr, pat, flags=0):
+    """
+    Find groups in each string using passed regular expression
 
     Parameters
     ----------
@@ -300,18 +383,47 @@ def str_match(arr, pat, flags=0):
 
     Returns
     -------
-    matches : array
+    extracted groups : Series (one group) or DataFrame (multiple groups)
+
+
+    Notes
+    -----
+    Compare to the string method match, which returns re.match objects.
     """
     regex = re.compile(pat, flags=flags)
 
-    def f(x):
-        m = regex.match(x)
-        if m:
-            return m.groups()
-        else:
-            return []
+    # just to be safe, check this
+    if regex.groups == 0:
+        raise ValueError("This pattern contains no groups to capture.")
+    elif regex.groups == 1:
+        def f(x):
+            if not isinstance(x, compat.string_types):
+                return None
+            m = regex.search(x)
+            if m:
+                return m.groups()[0]  # may be None
+            else:
+                return None
+    else:
+        empty_row = Series(regex.groups * [None])
 
-    return _na_map(f, arr)
+        def f(x):
+            if not isinstance(x, compat.string_types):
+                return empty_row
+            m = regex.search(x)
+            if m:
+                return Series(list(m.groups()))  # may contain None
+            else:
+                return empty_row
+    result = arr.apply(f)
+    result.replace({None: np.nan}, inplace=True)
+    if regex.groups > 1:
+        result = DataFrame(result)  # Don't rely on the wrapper; name columns.
+        names = dict(zip(regex.groupindex.values(), regex.groupindex.keys()))
+        result.columns = [names.get(1 + i, i) for i in range(regex.groups)]
+    else:
+        result.name = regex.groupindex.get(0)
+    return result
 
 
 def str_join(arr, sep):
@@ -549,7 +661,7 @@ def str_get(arr, i):
     -------
     items : array
     """
-    f = lambda x: x[i]
+    f = lambda x: x[i] if len(x) > i else np.nan
     return _na_map(f, arr)
 
 
@@ -599,13 +711,13 @@ def _noarg_wrapper(f):
     return wrapper
 
 
-def _pat_wrapper(f, flags=False, na=False):
+def _pat_wrapper(f, flags=False, na=False, **kwargs):
     def wrapper1(self, pat):
         result = f(self.series, pat)
         return self._wrap_result(result)
 
-    def wrapper2(self, pat, flags=0):
-        result = f(self.series, pat, flags=flags)
+    def wrapper2(self, pat, flags=0, **kwargs):
+        result = f(self.series, pat, flags=flags, **kwargs)
         return self._wrap_result(result)
 
     def wrapper3(self, pat, na=np.nan):
@@ -631,6 +743,7 @@ def copy(source):
 
 
 class StringMethods(object):
+
     """
     Vectorized string functions for Series. NAs stay NA unless handled
     otherwise by a particular method. Patterned after Python's string methods,
@@ -641,6 +754,7 @@ class StringMethods(object):
     >>> s.str.split('_')
     >>> s.str.replace('_', '')
     """
+
     def __init__(self, series):
         self.series = series
 
@@ -651,9 +765,21 @@ class StringMethods(object):
         else:
             return self.get(key)
 
+    def __iter__(self):
+        i = 0
+        g = self.get(i)
+        while g.notnull().any():
+            yield g
+            i += 1
+            g = self.get(i)
+
     def _wrap_result(self, result):
-        return Series(result, index=self.series.index,
-                      name=self.series.name)
+        assert result.ndim < 3
+        if result.ndim == 1:
+            return Series(result, index=self.series.index,
+                          name=self.series.name)
+        else:
+            return DataFrame(result, index=self.series.index)
 
     @copy(str_cat)
     def cat(self, others=None, sep=None, na_rep=None):
@@ -678,12 +804,13 @@ class StringMethods(object):
     @copy(str_contains)
     def contains(self, pat, case=True, flags=0, na=np.nan):
         result = str_contains(self.series, pat, case=case, flags=flags,
-                              na=np.nan)
+                              na=na)
         return self._wrap_result(result)
 
     @copy(str_replace)
-    def replace(self, pat, repl, n=-1, case=True):
-        result = str_replace(self.series, pat, repl, n=n, case=case)
+    def replace(self, pat, repl, n=-1, case=True, flags=0):
+        result = str_replace(self.series, pat, repl, n=n, case=case,
+                             flags=flags)
         return self._wrap_result(result)
 
     @copy(str_repeat)
@@ -740,7 +867,9 @@ class StringMethods(object):
     endswith = _pat_wrapper(str_endswith, na=True)
     findall = _pat_wrapper(str_findall, flags=True)
     match = _pat_wrapper(str_match, flags=True)
+    extract = _pat_wrapper(str_extract, flags=True)
 
     len = _noarg_wrapper(str_len)
     lower = _noarg_wrapper(str_lower)
     upper = _noarg_wrapper(str_upper)
+    title = _noarg_wrapper(str_title)
