@@ -5,20 +5,21 @@ Module contains tools for collecting data from various remote sources
 """
 import warnings
 import tempfile
-import itertools
 import datetime as dt
-import urllib
 import time
 
 from collections import defaultdict
 
 import numpy as np
 
-from pandas.util.py3compat import StringIO, bytes_to_str
+from pandas.compat import(
+    StringIO, bytes_to_str, range, lrange, lmap, zip
+)
+import pandas.compat as compat
 from pandas import Panel, DataFrame, Series, read_csv, concat
-from pandas.core.common import PandasError
+from pandas.core.common import is_list_like, PandasError
 from pandas.io.parsers import TextParser
-from pandas.io.common import urlopen, ZipFile
+from pandas.io.common import urlopen, ZipFile, urlencode
 from pandas.util.testing import _network_error_classes
 
 
@@ -35,15 +36,16 @@ def DataReader(name, data_source=None, start=None, end=None,
     """
     Imports data from a number of online sources.
 
-    Currently supports Yahoo! finance, St. Louis FED (FRED), and Kenneth
-    French's data library.
+    Currently supports Yahoo! Finance, Google Finance, St. Louis FED (FRED)
+    and Kenneth French's data library.
 
     Parameters
     ----------
-    name : str
-        the name of the dataset
+    name : str or list of strs
+        the name of the dataset. Some data sources (yahoo, google, fred) will
+        accept a list of names.
     data_source: str
-        the data source ("yahoo", "fred", or "ff")
+        the data source ("yahoo", "google", "fred", or "ff")
     start : {datetime, None}
         left boundary for range (defaults to 1/1/2010)
     end : {datetime, None}
@@ -52,9 +54,12 @@ def DataReader(name, data_source=None, start=None, end=None,
     Examples
     ----------
 
-    # Data from Yahoo!
+    # Data from Yahoo! Finance
     gs = DataReader("GS", "yahoo")
 
+    # Data from Google Finance
+    aapl = DataReader("AAPL", "google")
+
     # Data from FRED
     vix = DataReader("VIXCLS", "fred")
 
@@ -95,26 +100,27 @@ def _in_chunks(seq, size):
     """
     Return sequence in 'chunks' of size defined by size
     """
-    return (seq[pos:pos + size] for pos in xrange(0, len(seq), size))
+    return (seq[pos:pos + size] for pos in range(0, len(seq), size))
 
 
 _yahoo_codes = {'symbol': 's', 'last': 'l1', 'change_pct': 'p2', 'PE': 'r',
                 'time': 't1', 'short_ratio': 's7'}
 
+
 def get_quote_yahoo(symbols):
     """
     Get current yahoo quote
 
     Returns a DataFrame
     """
-    if isinstance(symbols, basestring):
+    if isinstance(symbols, compat.string_types):
         sym_list = symbols
     else:
         sym_list = '+'.join(symbols)
 
     # for codes see: http://www.gummy-stuff.org/Yahoo-data.htm
-    request = ''.join(_yahoo_codes.itervalues())  # code request string
-    header = _yahoo_codes.keys()
+    request = ''.join(compat.itervalues(_yahoo_codes))  # code request string
+    header = list(_yahoo_codes.keys())
 
     data = defaultdict(list)
 
@@ -147,7 +153,7 @@ def get_quote_google(symbols):
 
 
 def _retry_read_url(url, retry_count, pause, name):
-    for _ in xrange(retry_count):
+    for _ in range(retry_count):
         time.sleep(pause)
 
         # kludge to close the socket ASAP
@@ -201,11 +207,10 @@ def _get_hist_google(sym, start, end, retry_count, pause):
     google_URL = 'http://www.google.com/finance/historical?'
 
     # www.google.com/finance/historical?q=GOOG&startdate=Jun+9%2C+2011&enddate=Jun+8%2C+2013&output=csv
-    url = google_URL + urllib.urlencode({"q": sym,
-                                         "startdate": start.strftime('%b %d, '
-                                                                     '%Y'),
-                                         "enddate": end.strftime('%b %d, %Y'),
-                                         "output": "csv"})
+    url = google_URL + urlencode({"q": sym,
+                                  "startdate": start.strftime('%b %d, ' '%Y'),
+                                  "enddate": end.strftime('%b %d, %Y'),
+                                  "output": "csv"})
     return _retry_read_url(url, retry_count, pause, 'Google')
 
 
@@ -322,6 +327,7 @@ def _dl_mult_symbols(symbols, start, end, chunksize, retry_count, pause,
 
 _source_functions = {'google': _get_hist_google, 'yahoo': _get_hist_yahoo}
 
+
 def _get_data_from(symbols, start, end, retry_count, pause, adjust_price,
                    ret_index, chunksize, source, name):
     if name is not None:
@@ -332,7 +338,7 @@ def _get_data_from(symbols, start, end, retry_count, pause, adjust_price,
     src_fn = _source_functions[source]
 
     # If a single symbol, (e.g., 'GOOG')
-    if isinstance(symbols, (basestring, int)):
+    if isinstance(symbols, (compat.string_types, int)):
         hist_data = src_fn(symbols, start, end, retry_count, pause)
     # Or multiple symbols, (e.g., ['GOOG', 'AAPL', 'MSFT'])
     elif isinstance(symbols, DataFrame):
@@ -431,23 +437,37 @@ def get_data_fred(name, start=dt.datetime(2010, 1, 1),
     Date format is datetime
 
     Returns a DataFrame.
+
+    If multiple names are passed for "series" then the index of the
+    DataFrame is the outer join of the indicies of each series.
     """
     start, end = _sanitize_dates(start, end)
 
     fred_URL = "http://research.stlouisfed.org/fred2/series/"
 
-    url = fred_URL + '%s' % name + '/downloaddata/%s' % name + '.csv'
-    with urlopen(url) as resp:
-        data = read_csv(resp, index_col=0, parse_dates=True,
-                        header=None, skiprows=1, names=["DATE", name],
-                        na_values='.')
-    try:
-        return data.truncate(start, end)
-    except KeyError:
-        if data.ix[3].name[7:12] == 'Error':
-            raise IOError("Failed to get the data. Check that {0!r} is "
-                          "a valid FRED series.".format(name))
-        raise
+    if not is_list_like(name):
+        names = [name]
+    else:
+        names = name
+
+    urls = [fred_URL + '%s' % n + '/downloaddata/%s' % n + '.csv' for
+            n in names]
+
+    def fetch_data(url, name):
+        with urlopen(url) as resp:
+            data = read_csv(resp, index_col=0, parse_dates=True,
+                            header=None, skiprows=1, names=["DATE", name],
+                            na_values='.')
+        try:
+            return data.truncate(start, end)
+        except KeyError:
+            if data.ix[3].name[7:12] == 'Error':
+                raise IOError("Failed to get the data. Check that {0!r} is "
+                              "a valid FRED series.".format(name))
+            raise
+    df = concat([fetch_data(url, n) for url, n in zip(urls, names)],
+                axis=1, join='outer')
+    return df
 
 
 def get_data_famafrench(name):
@@ -465,15 +485,15 @@ def get_data_famafrench(name):
         with ZipFile(tmpf, 'r') as zf:
             data = zf.open(name + '.txt').readlines()
 
-    line_lengths = np.array(map(len, data))
+    line_lengths = np.array(lmap(len, data))
     file_edges = np.where(line_lengths == 2)[0]
 
     datasets = {}
-    edges = itertools.izip(file_edges + 1, file_edges[1:])
+    edges = zip(file_edges + 1, file_edges[1:])
     for i, (left_edge, right_edge) in enumerate(edges):
         dataset = [d.split() for d in data[left_edge:right_edge]]
         if len(dataset) > 10:
-            ncol_raw = np.array(map(len, dataset))
+            ncol_raw = np.array(lmap(len, dataset))
             ncol = np.median(ncol_raw)
             header_index = np.where(ncol_raw == ncol - 1)[0][-1]
             header = dataset[header_index]
@@ -809,18 +829,18 @@ class Options(object):
         data : dict of str, DataFrame
         """
         warnings.warn("get_forward_data() is deprecated", FutureWarning)
-        in_months = xrange(CUR_MONTH, CUR_MONTH + months + 1)
+        in_months = lrange(CUR_MONTH, CUR_MONTH + months + 1)
         in_years = [CUR_YEAR] * (months + 1)
 
         # Figure out how many items in in_months go past 12
         to_change = 0
-        for i in xrange(months):
+        for i in range(months):
             if in_months[i] > 12:
                 in_months[i] -= 12
                 to_change += 1
 
         # Change the corresponding items in the in_years list.
-        for i in xrange(1, to_change + 1):
+        for i in range(1, to_change + 1):
             in_years[-i] += 1
 
         to_ret = Series({'calls': call, 'puts': put})
@@ -830,7 +850,7 @@ class Options(object):
         for name in to_ret:
             all_data = DataFrame()
 
-            for mon in xrange(months):
+            for mon in range(months):
                 m2 = in_months[mon]
                 y2 = in_years[mon]
 
