@@ -4,7 +4,7 @@ from datetime import datetime, timedelta, date
 import os
 import operator
 import re
-import unittest
+import warnings
 
 import nose
 
@@ -13,6 +13,8 @@ import numpy as np
 from numpy.testing import assert_array_equal
 from numpy.random import randint
 
+from pandas.compat import range, lrange, u
+import pandas.compat as compat
 from pandas import (Index, Series, TimeSeries, DataFrame, isnull, notnull,
                     bdate_range, date_range)
 import pandas.core.common as com
@@ -23,7 +25,7 @@ import pandas.util.testing as tm
 import pandas.core.strings as strings
 
 
-class TestStringMethods(unittest.TestCase):
+class TestStringMethods(tm.TestCase):
 
     _multiprocess_can_split_ = True
 
@@ -34,15 +36,15 @@ class TestStringMethods(unittest.TestCase):
 
         for s in ds.str:
             # iter must yield a Series
-            self.assert_(isinstance(s, Series))
+            tm.assert_isinstance(s, Series)
 
             # indices of each yielded Series should be equal to the index of
             # the original Series
             assert_array_equal(s.index, ds.index)
 
             for el in s:
-                # each element of the series is either a basestring or nan
-                self.assert_(isinstance(el, basestring) or isnull(el))
+                # each element of the series is either a basestring/str or nan
+                self.assert_(isinstance(el, compat.string_types) or isnull(el))
 
         # desired behavior is to iterate until everything would be nan on the
         # next iter so make sure the last element of the iterator was 'l' in
@@ -73,7 +75,7 @@ class TestStringMethods(unittest.TestCase):
 
     def test_iter_numeric_try_string(self):
         # behavior identical to empty series
-        dsi = Series(range(4))
+        dsi = Series(lrange(4))
 
         i, s = 100, 'h'
 
@@ -93,7 +95,7 @@ class TestStringMethods(unittest.TestCase):
 
     def test_iter_object_try_string(self):
         ds = Series([slice(None, randint(10), randint(10, 20))
-                     for _ in xrange(4)])
+                     for _ in range(4)])
 
         i, s = 100, 'h'
 
@@ -140,7 +142,7 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(result, exp)
 
         result = Series(values).str.count('f[o]+')
-        self.assert_(isinstance(result, Series))
+        tm.assert_isinstance(result, Series)
         tm.assert_almost_equal(result, exp)
 
         # mixed
@@ -150,18 +152,18 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.count('a')
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = [u'foo', u'foofoo', NA, u'foooofooofommmfoo']
+        values = [u('foo'), u('foofoo'), NA, u('foooofooofommmfoo')]
 
         result = strings.str_count(values, 'f[o]+')
         exp = [1, 2, NA, 4]
         tm.assert_almost_equal(result, exp)
 
         result = Series(values).str.count('f[o]+')
-        self.assert_(isinstance(result, Series))
+        tm.assert_isinstance(result, Series)
         tm.assert_almost_equal(result, exp)
 
     def test_contains(self):
@@ -185,11 +187,11 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.contains('o')
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = [u'foo', NA, u'fooommm__foo', u'mmm_']
+        values = [u('foo'), NA, u('fooommm__foo'), u('mmm_')]
         pat = 'mmm[_]+'
 
         result = strings.str_contains(values, pat)
@@ -225,12 +227,12 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.startswith('f')
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'om', NA, u'foo_nom', u'nom', u'bar_foo', NA,
-                         u'foo'])
+        values = Series([u('om'), NA, u('foo_nom'), u('nom'), u('bar_foo'), NA,
+                         u('foo')])
 
         result = values.str.startswith('foo')
         exp = Series([False, NA, True, False, False, NA, True])
@@ -253,12 +255,12 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.endswith('f')
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'om', NA, u'foo_nom', u'nom', u'bar_foo', NA,
-                         u'foo'])
+        values = Series([u('om'), NA, u('foo_nom'), u('nom'), u('bar_foo'), NA,
+                         u('foo')])
 
         result = values.str.endswith('foo')
         exp = Series([False, NA, False, False, True, NA, True])
@@ -282,10 +284,10 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_almost_equal(mixed, exp)
 
         # unicode
-        values = Series([u"FOO", NA, u"bar", u"Blurg"])
+        values = Series([u("FOO"), NA, u("bar"), u("Blurg")])
 
         results = values.str.title()
-        exp = Series([u"Foo", NA, u"Bar", u"Blurg"])
+        exp = Series([u("Foo"), NA, u("Bar"), u("Blurg")])
 
         tm.assert_series_equal(results, exp)
 
@@ -305,14 +307,14 @@ class TestStringMethods(unittest.TestCase):
         mixed = mixed.str.upper()
         rs = Series(mixed).str.lower()
         xp = ['a', NA, 'b', NA, NA, 'foo', NA, NA, NA]
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'om', NA, u'nom', u'nom'])
+        values = Series([u('om'), NA, u('nom'), u('nom')])
 
         result = values.str.upper()
-        exp = Series([u'OM', NA, u'NOM', u'NOM'])
+        exp = Series([u('OM'), NA, u('NOM'), u('NOM')])
         tm.assert_series_equal(result, exp)
 
         result = result.str.lower()
@@ -335,18 +337,18 @@ class TestStringMethods(unittest.TestCase):
 
         rs = Series(mixed).str.replace('BAD[_]*', '')
         xp = ['a', NA, 'b', NA, NA, 'foo', NA, NA, NA]
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'fooBAD__barBAD', NA])
+        values = Series([u('fooBAD__barBAD'), NA])
 
         result = values.str.replace('BAD[_]*', '')
-        exp = Series([u'foobar', NA])
+        exp = Series([u('foobar'), NA])
         tm.assert_series_equal(result, exp)
 
         result = values.str.replace('BAD[_]*', '', n=1)
-        exp = Series([u'foobarBAD', NA])
+        exp = Series([u('foobarBAD'), NA])
         tm.assert_series_equal(result, exp)
 
         #flags + unicode
@@ -373,24 +375,29 @@ class TestStringMethods(unittest.TestCase):
 
         rs = Series(mixed).str.repeat(3)
         xp = ['aaa', NA, 'bbb', NA, NA, 'foofoofoo', NA, NA, NA]
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a', u'b', NA, u'c', NA, u'd'])
+        values = Series([u('a'), u('b'), NA, u('c'), NA,
+                         u('d')])
 
         result = values.str.repeat(3)
-        exp = Series([u'aaa', u'bbb', NA, u'ccc', NA, u'ddd'])
+        exp = Series([u('aaa'), u('bbb'), NA, u('ccc'), NA,
+                      u('ddd')])
         tm.assert_series_equal(result, exp)
 
         result = values.str.repeat([1, 2, 3, 4, 5, 6])
-        exp = Series([u'a', u'bb', NA, u'cccc', NA, u'dddddd'])
+        exp = Series([u('a'), u('bb'), NA, u('cccc'), NA,
+                      u('dddddd')])
         tm.assert_series_equal(result, exp)
 
-    def test_match(self):
+    def test_deprecated_match(self):
+        # Old match behavior, deprecated (but still default) in 0.13
         values = Series(['fooBAD__barBAD', NA, 'foo'])
 
-        result = values.str.match('.*(BAD[_]+).*(BAD)')
+        with tm.assert_produces_warning():
+            result = values.str.match('.*(BAD[_]+).*(BAD)')
         exp = Series([('BAD__', 'BAD'), NA, []])
         tm.assert_series_equal(result, exp)
 
@@ -398,18 +405,141 @@ class TestStringMethods(unittest.TestCase):
         mixed = Series(['aBAD_BAD', NA, 'BAD_b_BAD', True, datetime.today(),
                         'foo', None, 1, 2.])
 
-        rs = Series(mixed).str.match('.*(BAD[_]+).*(BAD)')
+        with tm.assert_produces_warning():
+            rs = Series(mixed).str.match('.*(BAD[_]+).*(BAD)')
         xp = [('BAD_', 'BAD'), NA, ('BAD_', 'BAD'), NA, NA, [], NA, NA, NA]
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'fooBAD__barBAD', NA, u'foo'])
+        values = Series([u('fooBAD__barBAD'), NA, u('foo')])
 
-        result = values.str.match('.*(BAD[_]+).*(BAD)')
-        exp = Series([(u'BAD__', u'BAD'), NA, []])
+        with tm.assert_produces_warning():
+            result = values.str.match('.*(BAD[_]+).*(BAD)')
+        exp = Series([(u('BAD__'), u('BAD')), NA, []])
         tm.assert_series_equal(result, exp)
 
+    def test_match(self):
+        # New match behavior introduced in 0.13
+        values = Series(['fooBAD__barBAD', NA, 'foo'])
+        with tm.assert_produces_warning():
+            result = values.str.match('.*(BAD[_]+).*(BAD)', as_indexer=True)
+        exp = Series([True, NA, False])
+        tm.assert_series_equal(result, exp)
+
+        # If no groups, use new behavior even when as_indexer is False.
+        # (Old behavior is pretty much useless in this case.)
+        values = Series(['fooBAD__barBAD', NA, 'foo'])
+        result = values.str.match('.*BAD[_]+.*BAD', as_indexer=False)
+        exp = Series([True, NA, False])
+        tm.assert_series_equal(result, exp)
+
+        # mixed
+        mixed = Series(['aBAD_BAD', NA, 'BAD_b_BAD', True, datetime.today(),
+                        'foo', None, 1, 2.])
+
+        with tm.assert_produces_warning():
+            rs = Series(mixed).str.match('.*(BAD[_]+).*(BAD)', as_indexer=True)
+        xp = [True, NA, True, NA, NA, False, NA, NA, NA]
+        tm.assert_isinstance(rs, Series)
+        tm.assert_almost_equal(rs, xp)
+
+        # unicode
+        values = Series([u('fooBAD__barBAD'), NA, u('foo')])
+
+        with tm.assert_produces_warning():
+            result = values.str.match('.*(BAD[_]+).*(BAD)', as_indexer=True)
+        exp = Series([True, NA, False])
+        tm.assert_series_equal(result, exp)
+
+    def test_extract(self):
+        # Contains tests like those in test_match and some others.
+
+        values = Series(['fooBAD__barBAD', NA, 'foo'])
+        er = [NA, NA] # empty row
+
+        result = values.str.extract('.*(BAD[_]+).*(BAD)')
+        exp = DataFrame([['BAD__', 'BAD'], er, er])
+        tm.assert_frame_equal(result, exp)
+
+        # mixed
+        mixed = Series(['aBAD_BAD', NA, 'BAD_b_BAD', True, datetime.today(),
+                        'foo', None, 1, 2.])
+
+        rs = Series(mixed).str.extract('.*(BAD[_]+).*(BAD)')
+        exp = DataFrame([['BAD_', 'BAD'], er, ['BAD_', 'BAD'], er, er,
+                         er, er, er, er])
+        tm.assert_frame_equal(rs, exp)
+
+        # unicode
+        values = Series([u('fooBAD__barBAD'), NA, u('foo')])
+
+        result = values.str.extract('.*(BAD[_]+).*(BAD)')
+        exp = DataFrame([[u('BAD__'), u('BAD')], er, er])
+        tm.assert_frame_equal(result, exp)
+
+        # no groups
+        s = Series(['A1', 'B2', 'C3'])
+        f = lambda: s.str.extract('[ABC][123]')
+        self.assertRaises(ValueError, f)
+
+        # only non-capturing groups
+        f = lambda: s.str.extract('(?:[AB]).*')
+        self.assertRaises(ValueError, f)
+
+        # one group, no matches
+        result = s.str.extract('(_)')
+        exp = Series([NA, NA, NA])
+        tm.assert_series_equal(result, exp)
+
+        # two groups, no matches
+        result = s.str.extract('(_)(_)')
+        exp = DataFrame([[NA, NA], [NA, NA], [NA, NA]])
+        tm.assert_frame_equal(result, exp)
+
+        # one group, some matches
+        result = s.str.extract('([AB])[123]')
+        exp = Series(['A', 'B', NA])
+        tm.assert_series_equal(result, exp)
+
+        # two groups, some matches
+        result = s.str.extract('([AB])([123])')
+        exp = DataFrame([['A', '1'], ['B', '2'], [NA, NA]])
+        tm.assert_frame_equal(result, exp)
+
+        # named group/groups
+        result = s.str.extract('(?P<letter>[AB])(?P<number>[123])')
+        exp = DataFrame([['A', '1'], ['B', '2'], [NA, NA]], columns=['letter', 'number'])
+        tm.assert_frame_equal(result, exp)
+        result = s.str.extract('(?P<letter>[AB])')
+        exp = Series(['A', 'B', NA], name='letter')
+        tm.assert_series_equal(result, exp)
+
+        # mix named and unnamed groups
+        result = s.str.extract('([AB])(?P<number>[123])')
+        exp = DataFrame([['A', '1'], ['B', '2'], [NA, NA]], columns=[0, 'number'])
+        tm.assert_frame_equal(result, exp)
+
+        # one normal group, one non-capturing group
+        result = s.str.extract('([AB])(?:[123])')
+        exp = Series(['A', 'B', NA])
+        tm.assert_series_equal(result, exp)
+
+        # two normal groups, one non-capturing group
+        result = Series(['A11', 'B22', 'C33']).str.extract('([AB])([123])(?:[123])')
+        exp = DataFrame([['A', '1'], ['B', '2'], [NA, NA]])
+        tm.assert_frame_equal(result, exp)
+
+        # one optional group followed by one normal group
+        result = Series(['A1', 'B2', '3']).str.extract('(?P<letter>[AB])?(?P<number>[123])')
+        exp = DataFrame([['A', '1'], ['B', '2'], [NA, '3']], columns=['letter', 'number'])
+        tm.assert_frame_equal(result, exp)
+
+        # one normal group followed by one optional group
+        result = Series(['A1', 'B2', 'C']).str.extract('(?P<letter>[ABC])(?P<number>[123])?')
+        exp = DataFrame([['A', '1'], ['B', '2'], ['C', NA]], columns=['letter', 'number'])
+        tm.assert_frame_equal(result, exp)
+
     def test_join(self):
         values = Series(['a_b_c', 'c_d_e', np.nan, 'f_g_h'])
         result = values.str.split('_').str.join('_')
@@ -422,11 +552,12 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.split('_').str.join('_')
         xp = Series(['a_b', NA, 'asdf_cas_asdf', NA, NA, 'foo', NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a_b_c', u'c_d_e', np.nan, u'f_g_h'])
+        values = Series([u('a_b_c'), u('c_d_e'), np.nan,
+                         u('f_g_h')])
         result = values.str.split('_').str.join('_')
         tm.assert_series_equal(values, result)
 
@@ -444,11 +575,12 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.len()
         xp = Series([3, NA, 13, NA, NA, 3, NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'foo', u'fooo', u'fooooo', np.nan, u'fooooooo'])
+        values = Series([u('foo'), u('fooo'), u('fooooo'), np.nan,
+                         u('fooooooo')])
 
         result = values.str.len()
         exp = values.map(lambda x: len(x) if com.notnull(x) else NA)
@@ -468,14 +600,15 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.findall('BAD[_]*')
         xp = Series([['BAD__', 'BAD'], NA, [], NA, NA, ['BAD'], NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'fooBAD__barBAD', NA, u'foo', u'BAD'])
+        values = Series([u('fooBAD__barBAD'), NA, u('foo'),
+                         u('BAD')])
 
         result = values.str.findall('BAD[_]*')
-        exp = Series([[u'BAD__', u'BAD'], NA, [], [u'BAD']])
+        exp = Series([[u('BAD__'), u('BAD')], NA, [], [u('BAD')]])
         tm.assert_almost_equal(result, exp)
 
     def test_pad(self):
@@ -500,7 +633,7 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.pad(5, side='left')
         xp = Series(['    a', NA, '    b', NA, NA, '   ee', NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         mixed = Series(['a', NA, 'b', True, datetime.today(),
@@ -509,7 +642,7 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.pad(5, side='right')
         xp = Series(['a    ', NA, 'b    ', NA, NA, 'ee   ', NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         mixed = Series(['a', NA, 'b', True, datetime.today(),
@@ -518,22 +651,26 @@ class TestStringMethods(unittest.TestCase):
         rs = Series(mixed).str.pad(5, side='both')
         xp = Series(['  a  ', NA, '  b  ', NA, NA, '  ee ', NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a', u'b', NA, u'c', NA, u'eeeeee'])
+        values = Series([u('a'), u('b'), NA, u('c'), NA,
+                         u('eeeeee')])
 
         result = values.str.pad(5, side='left')
-        exp = Series([u'    a', u'    b', NA, u'    c', NA, u'eeeeee'])
+        exp = Series([u('    a'), u('    b'), NA, u('    c'), NA,
+                      u('eeeeee')])
         tm.assert_almost_equal(result, exp)
 
         result = values.str.pad(5, side='right')
-        exp = Series([u'a    ', u'b    ', NA, u'c    ', NA, u'eeeeee'])
+        exp = Series([u('a    '), u('b    '), NA, u('c    '), NA,
+                      u('eeeeee')])
         tm.assert_almost_equal(result, exp)
 
         result = values.str.pad(5, side='both')
-        exp = Series([u'  a  ', u'  b  ', NA, u'  c  ', NA, u'eeeeee'])
+        exp = Series([u('  a  '), u('  b  '), NA, u('  c  '), NA,
+                      u('eeeeee')])
         tm.assert_almost_equal(result, exp)
 
     def test_center(self):
@@ -551,14 +688,16 @@ class TestStringMethods(unittest.TestCase):
         xp = Series(['  a  ', NA, '  b  ', NA, NA, '  c  ', ' eee ', NA, NA,
                      NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a', u'b', NA, u'c', NA, u'eeeeee'])
+        values = Series([u('a'), u('b'), NA, u('c'), NA,
+                         u('eeeeee')])
 
         result = values.str.center(5)
-        exp = Series([u'  a  ', u'  b  ', NA, u'  c  ', NA, u'eeeeee'])
+        exp = Series([u('  a  '), u('  b  '), NA, u('  c  '), NA,
+                      u('eeeeee')])
         tm.assert_almost_equal(result, exp)
 
     def test_split(self):
@@ -581,15 +720,16 @@ class TestStringMethods(unittest.TestCase):
         xp = Series([['a', 'b', 'c'], NA, ['d', 'e', 'f'], NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a_b_c', u'c_d_e', NA, u'f_g_h'])
+        values = Series([u('a_b_c'), u('c_d_e'), NA, u('f_g_h')])
 
         result = values.str.split('_')
-        exp = Series([[u'a', u'b', u'c'], [u'c', u'd', u'e'], NA,
-                      [u'f', u'g', u'h']])
+        exp = Series([[u('a'), u('b'), u('c')],
+                      [u('c'), u('d'), u('e')], NA,
+                      [u('f'), u('g'), u('h')]])
         tm.assert_series_equal(result, exp)
 
     def test_split_noargs(self):
@@ -646,14 +786,15 @@ class TestStringMethods(unittest.TestCase):
         xp = Series(['foo', NA, 'bar', NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'aafootwo', u'aabartwo', NA, u'aabazqux'])
+        values = Series([u('aafootwo'), u('aabartwo'), NA,
+                         u('aabazqux')])
 
         result = values.str.slice(2, 5)
-        exp = Series([u'foo', u'bar', NA, u'baz'])
+        exp = Series([u('foo'), u('bar'), NA, u('baz')])
         tm.assert_series_equal(result, exp)
 
     def test_slice_replace(self):
@@ -683,37 +824,38 @@ class TestStringMethods(unittest.TestCase):
         xp = Series(['aa', NA, 'bb', NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.lstrip()
         xp = Series(['aa  ', NA, 'bb \t\n', NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         rs = Series(mixed).str.rstrip()
         xp = Series(['  aa', NA, ' bb', NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
     def test_strip_lstrip_rstrip_unicode(self):
         # unicode
-        values = Series([u'  aa   ', u' bb \n', NA, u'cc  '])
+        values = Series([u('  aa   '), u(' bb \n'), NA,
+                         u('cc  ')])
 
         result = values.str.strip()
-        exp = Series([u'aa', u'bb', NA, u'cc'])
+        exp = Series([u('aa'), u('bb'), NA, u('cc')])
         tm.assert_series_equal(result, exp)
 
         result = values.str.lstrip()
-        exp = Series([u'aa   ', u'bb \n', NA, u'cc  '])
+        exp = Series([u('aa   '), u('bb \n'), NA, u('cc  ')])
         tm.assert_series_equal(result, exp)
 
         result = values.str.rstrip()
-        exp = Series([u'  aa', u' bb', NA, u'cc'])
+        exp = Series([u('  aa'), u(' bb'), NA, u('cc')])
         tm.assert_series_equal(result, exp)
 
     def test_strip_lstrip_rstrip_args(self):
@@ -732,17 +874,18 @@ class TestStringMethods(unittest.TestCase):
         assert_series_equal(rs, xp)
 
     def test_strip_lstrip_rstrip_args_unicode(self):
-        values = Series([u'xxABCxx', u'xx BNSD', u'LDFJH xx'])
+        values = Series([u('xxABCxx'), u('xx BNSD'),
+                         u('LDFJH xx')])
 
-        rs = values.str.strip(u'x')
+        rs = values.str.strip(u('x'))
         xp = Series(['ABC', ' BNSD', 'LDFJH '])
         assert_series_equal(rs, xp)
 
-        rs = values.str.lstrip(u'x')
+        rs = values.str.lstrip(u('x'))
         xp = Series(['ABCxx', ' BNSD', 'LDFJH xx'])
         assert_series_equal(rs, xp)
 
-        rs = values.str.rstrip(u'x')
+        rs = values.str.rstrip(u('x'))
         xp = Series(['xxABC', 'xx BNSD', 'LDFJH '])
         assert_series_equal(rs, xp)
 
@@ -764,14 +907,15 @@ class TestStringMethods(unittest.TestCase):
         xp = Series(['b', NA, 'd', NA, NA,
                      NA, NA, NA])
 
-        self.assert_(isinstance(rs, Series))
+        tm.assert_isinstance(rs, Series)
         tm.assert_almost_equal(rs, xp)
 
         # unicode
-        values = Series([u'a_b_c', u'c_d_e', np.nan, u'f_g_h'])
+        values = Series([u('a_b_c'), u('c_d_e'), np.nan,
+                         u('f_g_h')])
 
         result = values.str.split('_').str.get(1)
-        expected = Series([u'b', u'd', np.nan, u'g'])
+        expected = Series([u('b'), u('d'), np.nan, u('g')])
         tm.assert_series_equal(result, expected)
 
     def test_more_contains(self):
@@ -859,7 +1003,10 @@ class TestStringMethods(unittest.TestCase):
 
         pat = pattern = r'([A-Z0-9._%+-]+)@([A-Z0-9.-]+)\.([A-Z]{2,4})'
 
-        result = data.str.match(pat, flags=re.IGNORECASE)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            result = data.str.match(pat, flags=re.IGNORECASE)
+            assert issubclass(w[-1].category, UserWarning)
         self.assertEquals(result[0], ('dave', 'google', 'com'))
 
         result = data.str.findall(pat, flags=re.IGNORECASE)
@@ -868,11 +1015,14 @@ class TestStringMethods(unittest.TestCase):
         result = data.str.count(pat, flags=re.IGNORECASE)
         self.assertEquals(result[0], 1)
 
-        result = data.str.contains(pat, flags=re.IGNORECASE)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            result = data.str.contains(pat, flags=re.IGNORECASE)
+            assert issubclass(w[-1].category, UserWarning)
         self.assertEquals(result[0], True)
 
     def test_encode_decode(self):
-        base = Series([u'a', u'b', u'a\xe4'])
+        base = Series([u('a'), u('b'), u('a\xe4')])
         series = base.str.encode('utf-8')
 
         f = lambda x: x.decode('utf-8')
@@ -882,7 +1032,7 @@ class TestStringMethods(unittest.TestCase):
         tm.assert_series_equal(result, exp)
 
     def test_encode_decode_errors(self):
-        encodeBase = Series([u'a', u'b', u'a\x9d'])
+        encodeBase = Series([u('a'), u('b'), u('a\x9d')])
 
         self.assertRaises(UnicodeEncodeError,
                           encodeBase.str.encode, 'cp1252')
