@@ -1,16 +1,15 @@
-import unittest
 
 from numpy import nan
 import numpy as np
-from pandas import Index, isnull
+from pandas import Index, isnull, Timestamp
 from pandas.util.testing import assert_almost_equal
-import pandas.util.testing as common
+import pandas.util.testing as tm
+from pandas.compat import range, lrange, zip
 import pandas.lib as lib
 import pandas.algos as algos
 from datetime import datetime
 
-
-class TestTseriesUtil(unittest.TestCase):
+class TestTseriesUtil(tm.TestCase):
     _multiprocess_can_split_ = True
 
     def test_combineFunc(self):
@@ -30,7 +29,7 @@ class TestTseriesUtil(unittest.TestCase):
 
     def test_backfill(self):
         old = Index([1, 5, 10])
-        new = Index(range(12))
+        new = Index(lrange(12))
 
         filler = algos.backfill_int64(old, new)
 
@@ -39,7 +38,7 @@ class TestTseriesUtil(unittest.TestCase):
 
         # corner case
         old = Index([1, 4])
-        new = Index(range(5, 10))
+        new = Index(lrange(5, 10))
         filler = algos.backfill_int64(old, new)
 
         expect_filler = [-1, -1, -1, -1, -1]
@@ -47,7 +46,7 @@ class TestTseriesUtil(unittest.TestCase):
 
     def test_pad(self):
         old = Index([1, 5, 10])
-        new = Index(range(12))
+        new = Index(lrange(12))
 
         filler = algos.pad_int64(old, new)
 
@@ -56,7 +55,7 @@ class TestTseriesUtil(unittest.TestCase):
 
         # corner case
         old = Index([5, 10])
-        new = Index(range(5))
+        new = Index(lrange(5))
         filler = algos.pad_int64(old, new)
         expect_filler = [-1, -1, -1, -1, -1]
         self.assert_(np.array_equal(filler, expect_filler))
@@ -420,7 +419,7 @@ def test_series_bin_grouper():
     assert_almost_equal(counts, exp_counts)
 
 
-class TestBinGroupers(unittest.TestCase):
+class TestBinGroupers(tm.TestCase):
     _multiprocess_can_split_ = True
 
     def setUp(self):
@@ -458,7 +457,9 @@ class TestBinGroupers(unittest.TestCase):
                           values, [-3, -1], 'right')
 
     def test_group_bin_functions(self):
-        funcs = ['add', 'mean', 'prod', 'min', 'max', 'var']
+
+        dtypes = ['float32','float64']
+        funcs  = ['add', 'mean', 'prod', 'min', 'max', 'var']
 
         np_funcs = {
             'add': np.sum,
@@ -470,71 +471,82 @@ class TestBinGroupers(unittest.TestCase):
         }
 
         for fname in funcs:
-            args = [getattr(algos, 'group_%s' % fname),
-                    getattr(algos, 'group_%s_bin' % fname),
-                    np_funcs[fname]]
-            self._check_versions(*args)
+            for d in dtypes:
+                check_less_precise = False
+                if d == 'float32':
+                    check_less_precise = True
+                args = [getattr(algos, 'group_%s_%s' % (fname,d)),
+                        getattr(algos, 'group_%s_bin_%s' % (fname,d)),
+                        np_funcs[fname],
+                        d,
+                        check_less_precise]
+                self._check_versions(*args)
 
-    def _check_versions(self, irr_func, bin_func, np_func):
-        obj = self.obj
+    def _check_versions(self, irr_func, bin_func, np_func, dtype, check_less_precise):
+        obj = self.obj.astype(dtype)
 
         cts = np.zeros(3, dtype=np.int64)
-        exp = np.zeros((3, 1), np.float64)
+        exp = np.zeros((3, 1), dtype)
         irr_func(exp, cts, obj, self.labels)
 
         # bin-based version
         bins = np.array([3, 6], dtype=np.int64)
-        out = np.zeros((3, 1), np.float64)
+        out = np.zeros((3, 1), dtype)
         counts = np.zeros(len(out), dtype=np.int64)
         bin_func(out, counts, obj, bins)
 
-        assert_almost_equal(out, exp)
+        assert_almost_equal(out, exp, check_less_precise=check_less_precise)
 
         bins = np.array([3, 9, 10], dtype=np.int64)
-        out = np.zeros((3, 1), np.float64)
+        out = np.zeros((3, 1), dtype)
         counts = np.zeros(len(out), dtype=np.int64)
         bin_func(out, counts, obj, bins)
         exp = np.array([np_func(obj[:3]), np_func(obj[3:9]),
                         np_func(obj[9:])],
-                       dtype=np.float64)
-        assert_almost_equal(out.squeeze(), exp)
+                       dtype=dtype)
+        assert_almost_equal(out.squeeze(), exp, check_less_precise=check_less_precise)
 
         # duplicate bins
         bins = np.array([3, 6, 10, 10], dtype=np.int64)
-        out = np.zeros((4, 1), np.float64)
+        out = np.zeros((4, 1), dtype)
         counts = np.zeros(len(out), dtype=np.int64)
         bin_func(out, counts, obj, bins)
         exp = np.array([np_func(obj[:3]), np_func(obj[3:6]),
                         np_func(obj[6:10]), np.nan],
-                       dtype=np.float64)
-        assert_almost_equal(out.squeeze(), exp)
+                       dtype=dtype)
+        assert_almost_equal(out.squeeze(), exp, check_less_precise=check_less_precise)
 
 
 def test_group_ohlc():
-    obj = np.random.randn(20)
 
-    bins = np.array([6, 12], dtype=np.int64)
-    out = np.zeros((3, 4), np.float64)
-    counts = np.zeros(len(out), dtype=np.int64)
+    def _check(dtype):
+        obj = np.array(np.random.randn(20),dtype=dtype)
 
-    algos.group_ohlc(out, counts, obj[:, None], bins)
+        bins = np.array([6, 12], dtype=np.int64)
+        out = np.zeros((3, 4), dtype)
+        counts = np.zeros(len(out), dtype=np.int64)
 
-    def _ohlc(group):
-        if isnull(group).all():
-            return np.repeat(nan, 4)
-        return [group[0], group.max(), group.min(), group[-1]]
+        func = getattr(algos,'group_ohlc_%s' % dtype)
+        func(out, counts, obj[:, None], bins)
 
-    expected = np.array([_ohlc(obj[:6]), _ohlc(obj[6:12]),
-                         _ohlc(obj[12:])])
+        def _ohlc(group):
+            if isnull(group).all():
+                return np.repeat(nan, 4)
+            return [group[0], group.max(), group.min(), group[-1]]
 
-    assert_almost_equal(out, expected)
-    assert_almost_equal(counts, [6, 6, 8])
+        expected = np.array([_ohlc(obj[:6]), _ohlc(obj[6:12]),
+                             _ohlc(obj[12:])])
 
-    obj[:6] = nan
-    algos.group_ohlc(out, counts, obj[:, None], bins)
-    expected[0] = nan
-    assert_almost_equal(out, expected)
+        assert_almost_equal(out, expected)
+        assert_almost_equal(counts, [6, 6, 8])
 
+        obj[:6] = nan
+        func(out, counts, obj[:, None], bins)
+        expected[0] = nan
+        assert_almost_equal(out, expected)
+
+    _check('float32')
+    _check('float64')
 
 def test_try_parse_dates():
     from dateutil.parser import parse
@@ -546,14 +558,14 @@ def test_try_parse_dates():
     assert(np.array_equal(result, expected))
 
 
-class TestTypeInference(unittest.TestCase):
+class TestTypeInference(tm.TestCase):
     _multiprocess_can_split_ = True
 
     def test_length_zero(self):
         result = lib.infer_dtype(np.array([], dtype='i4'))
-        self.assertEqual(result, 'empty')
+        self.assertEqual(result, 'integer')
 
-        result = lib.infer_dtype(np.array([], dtype='O'))
+        result = lib.infer_dtype([])
         self.assertEqual(result, 'empty')
 
     def test_integers(self):
@@ -639,11 +651,11 @@ class TestTypeInference(unittest.TestCase):
             pass
 
 
-class TestMoments(unittest.TestCase):
+class TestMoments(tm.TestCase):
     pass
 
 
-class TestReducer(unittest.TestCase):
+class TestReducer(tm.TestCase):
 
     def test_int_index(self):
         from pandas.core.series import Series
@@ -670,6 +682,32 @@ class TestReducer(unittest.TestCase):
         expected = arr.sum(1)
         assert_almost_equal(result, expected)
 
+
+class TestTsUtil(tm.TestCase):
+    def test_min_valid(self):
+        # Ensure that Timestamp.min is a valid Timestamp
+        Timestamp(Timestamp.min)
+
+    def test_max_valid(self):
+        # Ensure that Timestamp.max is a valid Timestamp
+        Timestamp(Timestamp.max)
+
+    def test_to_datetime_bijective(self):
+        # Ensure that converting to datetime and back only loses precision
+        # by going from nanoseconds to microseconds.
+        self.assertEqual(Timestamp(Timestamp.max.to_pydatetime()).value/1000, Timestamp.max.value/1000)
+        self.assertEqual(Timestamp(Timestamp.min.to_pydatetime()).value/1000, Timestamp.min.value/1000)
+
+class TestPeriodField(tm.TestCase):
+
+    def test_get_period_field_raises_on_out_of_range(self):
+        from pandas import tslib
+        self.assertRaises(ValueError, tslib.get_period_field, -1, 0, 0)
+
+    def test_get_period_field_array_raises_on_out_of_range(self):
+        from pandas import tslib
+        self.assertRaises(ValueError, tslib.get_period_field_arr, -1, np.empty(1), 0)
+
 if __name__ == '__main__':
     import nose
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
